program comparion_chebyshev

! Program for solving the stochastic growth model with labor using the weighted residual method with
! Chebyshev polynomials as the basis functions. 


!*************************************************************************************************************************

! Copyright  2003 S. Boragan Aruoba, Jesus Fernandez-Villaverde and Juan F. Rubio-Ramirez.

! This code is available on the World Wide Web at http://www.econ.upenn.edu/~jesusfv 
! and it is a part of the package prepared for the paper "Comparing Solution Methods for Dynamic Equilibrium Economies". 

! It may be freely reproduced for educational and research purposes, so long as it is not altered,
! this copyright notice is reproduced with it, and it is not sold for profit.

! Consent of the corresponding author (jesusfv@econ.upenn.edu) must be obtained before using all or any part of this code
! in a publication. 

! This version			: July 30, 2003

!************************************************************************************************************************	

	USE define_parameters

	USE solve_module

	USE small_functions

	implicit none

	external get_psi

	interface

		function residual(theta_in,neq)

			use define_parameters
			implicit none
			integer neq
			real(kind = real_kind), dimension(neq,1) :: theta_in
			real(kind = real_kind), dimension(neq,1) :: residual

		end function

	end interface
		
	!******* DEFINE VARIABLES ********	

	real(kind = real_kind), dimension(n*m,1) :: theta_0, theta_bench	! Starting Value for Theta
	real(kind = real_kind) fnorm_ee						! The norm of the residual function at the solution
	real(real_kind) :: one

	! To measure elapsed time	

	real(kind = real_kind) process_time
	real cpsec

	! For finding the steady state

	real(kind = real_kind) :: y_over_k, l_over_k, c_over_k, one_over_k

	! For solving the Decision Rules

	integer start_n, go_on, dim_temp, bench
	real(kind = real_kind), allocatable, dimension(:,:) :: theta_start, theta_temp

	! For the transition matrix

	real(real_kind), dimension( m * m, 1) :: z_prime_temp, transition_temp


	10	format(A,F8.2)   ! For printing

	! Error-Check

	if (n /2. == int(n/2)) then
		print*, 'ERROR : n has to be an odd number so that steady state capital will be'
		print*, 'one of the collocation points.'
		stop
	end if 

	! Open files for reading the matrices for the integration rule

	open(unit=5,file ='input_z.txt',form='formatted')
	open(unit=6,file ='input_z_prime.txt',form='formatted')	
	open(unit=7,file ='input_transition.txt',form='formatted')

	! Open the file for the benchmark theta

	open(unit=13,file='theta_bench.txt',form='formatted')

	! Open files for writing the results

	open(unit=11,file='results_param.txt',status='replace',form='formatted')
	open(unit=12,file='results_theta.txt',status='replace',form='formatted')	

	!******* INITIALIZE VARIABLES ********

	! Time

	call cpsec()

	pi = 4. * atan(1.)

	one = 1.

	!******* READ THE INTEGRATION RULE *********

	read (unit = 5, fmt='(E25.16)'), input_z
	read (unit = 6, fmt='(E25.16)'), z_prime_temp
	read (unit = 7, fmt='(E25.16)'), transition_temp
	close(unit=5)
	close(unit=6)
	close(unit=7)

	ZZ_PRIME = reshape(z_prime_temp , (/  m  ,  m  /))

	TRANSITION = reshape(transition_temp , (/  m  ,  m  /))

	!******** FIND THE STEADY STATE **********

	! From the intertemporal Euler Equation

	y_over_k = (((1.0d0 / beta1) - 1.0d0 + delta) / alpha)

	l_over_k = y_over_k ** (1.0d0 / (1.0d0 - alpha))

	! From the budget constraint

	c_over_k = y_over_k - delta

	! From the intratemporal Euler Equation

	l_ss = (theta * (1.0d0 - alpha)) / ( (1.0d0 - theta) * ( 1.0d0 - (delta * (1.0d0/y_over_k))) + theta * (1.0d0 - alpha) )

	k_ss = 1.0d0 / (l_over_k * (1.0d0/l_ss))

	y_ss = y_over_k * k_ss

	c_ss = c_over_k * k_ss

	! Compute the limits of the state space for capital

	k_min = (1.0d0 - cover_k) * k_ss
	k_max = (1.0d0 + cover_k) * k_ss	

	!******* START SOLVING FOR THE DECISION RULES ********

	! Find an Appropriate Starting Value
	
	print*, 'Solving for the decision rules.....'

	! Set bench = 1 to use the theta's in theta_bench.txt as starting point
	! Set go_on = 1 to use the iterative procedure. 
	
	bench = 0
	go_on = 1

	start_n = 3

	do while (go_on == 1)

		dim_temp = start_n * m

		allocate(theta_temp(start_n*m,1))

		if (start_n == 3) then			

			! Start with a horizontal line at l_ss for labor

			theta_temp = 0			
			theta_temp(1,:) = l_ss 						
				
			allocate(theta_start(dim_temp,1))
			
			theta_start = theta_temp			
		
		end if 

		n_iter = start_n

		call get_psi()

		call solve (residual, theta_start , theta_temp, start_n * m, eps , del , maxit, fnorm_ee, 1, one)

		print 10, 'The norm of the function at the solution is ', fnorm_ee

		start_n = start_n + 1

		deallocate(theta_start)

		deallocate(KK,ZZ,PSI_USE)

		allocate(theta_start(start_n*m,1))

		theta_start = 0

		theta_start(1:(start_n*m) - m,1) = theta_temp(1:(start_n*m)-m,1)
		
		if (start_n >= n .or. abs(theta_start((start_n-2)*m + 1,1)) < 1E-5) go_on = 0
		
		deallocate(theta_temp)
	end do 	

	if (bench == 1) then
		print*, 'Solving using last theta...'
		read (unit=13,fmt='(F25.15)'), theta_bench
		theta_0 = theta_bench
	else
	
		print*, 'Out of the iteration'
				
		theta_0 = 0.0d0

		theta_0(1 : (start_n - 1) * m,1) = theta_start(1 : (start_n - 1) * m,1)
	
		deallocate(theta_start)

	end if 

	n_iter = n

	call get_psi()

	call solve (residual , theta_0 , theta_weight,n * m, eps , del , maxit, fnorm_ee, 1, one)

	print 10, 'The norm of the function at the solution is ', fnorm_ee

	print*, 'Succesfully found a solution using the Collocation Method'

	process_time = cpsec()

	print 10, 'Processor time used for this program is  ', process_time, ' seconds.'

	print*, ' '
	print*, 'The parameters and solution of theta and v is written to the files'
	print*, 'results_param.txt, results_theta.txt and results_v.txt'

	! Write Results 

	write (unit=11,fmt='(I3)') n, m
	write (unit=11,fmt='(F20.16)'), k_min, k_max, z_min, z_max, alpha, beta1, delta, tau, theta, rho, sigma, k_ss, l_ss, y_ss, c_ss

	write (unit=12,fmt='(F20.16)'), theta_weight

	print*, ' '
	print*, 'The program will terminate now....'

end program comparion_chebyshev