module solve_module

!*************************************************************************************************************************

! Copyright  2003 S. Boragan Aruoba, Jesus Fernandez-Villaverde and Juan F. Rubio-Ramirez.

! This code is available on the World Wide Web at http://www.econ.upenn.edu/~jesusfv 
! and it is a part of the package prepared for the paper "Comparing Solution Methods for Dynamic Equilibrium Economies". 

! It may be freely reproduced for educational and research purposes, so long as it is not altered,
! this copyright notice is reproduced with it, and it is not sold for profit.

! Consent of the corresponding author (jesusfv@econ.upenn.edu) must be obtained before using all or any part of this code
! in a publication. 

! Note : This code includes parts from the Numerical Recipies package which can be obtained and distributed freely.

! This version			: September 3, 2003

!************************************************************************************************************************	


	implicit none

	! Ask for 15 digits of precision from all subroutines

	integer, parameter :: real_kind_solve = selected_real_kind(P=15)
	
	contains

subroutine solve (f , initial,final, n, eps , del , maxit, fnorm, print_options,step_size)

	! This subroutine solves for the zero of the function given by "f"
	! starting at guess given by initial. The result is written to final. 

	! eps   : Stopping criterion for x
	! delta : Convergence criterion for f(x)
	! maxit : Maximum number of iterations
	! fnorm : Norm of the function at solution

	implicit none

	! Define variables

	! Arguments
	integer, intent(in) :: n			! Size of the problem
	real(kind = real_kind_solve), dimension(n,1), intent(in) :: initial
	real(kind = real_kind_solve), dimension(n,1), intent(out) :: final		
	real(kind = real_kind_solve), intent(in) :: eps, del
	integer, intent(in) :: maxit
	integer, intent(in) :: print_options	
	real(kind = real_kind_solve), intent(in) :: step_size
	real(kind = real_kind_solve), intent(out) :: fnorm

	interface
		function f(theta_in,neq)			
			integer neq
			real(kind = selected_real_kind(P=15)), dimension(neq,1) :: theta_in
			real(kind = selected_real_kind(P=15)), dimension(neq,1) :: f
		end function
	end interface

	! Local Variables

	integer :: iter, flag1, flag2, exitflag
	real(kind = real_kind_solve), dimension(n,n) :: jacob, jacob_inv
	real(kind = real_kind_solve), dimension(n,1) :: func_eval, init, step
	real(kind = real_kind_solve) :: norm_step

	! For dludcmbp dlubskb

	real(kind = real_kind_solve) :: d
	real(kind = real_kind_solve), dimension(n) :: p
	integer, dimension(n) :: index

	! Format Statements
	
	10	format(A,E10.2)   ! For printing
	99 format(A , I5 , A , f20.16 , A , f20.16)

	! Start the iterations

	flag1 = 0
	flag2 = 0
	exitflag = 0

	func_eval = f(initial , n)

	init = initial

	final = init

	norm_step = 0.

	if (print_options == 1) print *, 'Iteration       Norm of relative step	             Norm of f(x)'
	
	do iter = 1 , maxit	

		fnorm = norm(func_eval,n,2)

		if ((mod(iter,1) == 0 .or. iter==1) .and. (print_options == 1)) then
			print 99,"   ", iter , "           " , norm_step , "                ", fnorm
		end if

		if ( fnorm < del ) then
			flag2 = 1
		end if

		if ( flag1 == 1) then	
			exitflag = 1
			exit
		end if		
		
		if ( flag2 == 1) then
			exitflag = 2
			exit
		end if

		jacob = jacobian(f ,init , n)		

		p = - func_eval(:,1)

		call dludcmp(jacob,index,d)
		call dlubksb(jacob,index,p)

		step(:,1) = p

		final = init + step_size * step

		! Check for convergence

		norm_step = norm( final - init,n,2) / (1 + norm(final,n,2))	! Could be sup or Euclidian norm

		if ( norm_step < eps) then
			flag1 = 1
		end if
	
		func_eval = f(final, n)			

		init = final	

	end do

	! Check if the solution satisfies the criterion
	
	if (print_options == 1) then
		if (exitflag == 1 .and. flag2 == 1) then
			print 10, 'Succesfully found a solution with norm ' , fnorm
		end if

		if (exitflag == 1 .and. flag2 == 0) then
			print 10, 'The problem converged to a point with norm ' , fnorm
		end if

		if (exitflag == 2) then
			print 10, 'Although x did not converge, f(x) < tol with norm ' ,fnorm
		end if
		
		if (exitflag == 0) then
			print 10, 'Maximum number of iterations reached with norm ', fnorm	
		end if
	end if

end subroutine

!******************************************

function jacobian(f , x , m)
	
	! Two-point finite difference approximation to the Jacobian

	implicit none

		interface

		function f(theta_in,neq)			
			
			integer neq
			real(kind = selected_real_kind(P=15)), dimension(neq,1) :: theta_in
			real(kind = selected_real_kind(P=15)), dimension(neq,1) :: f

		end function

	end interface

	integer :: m
	real(kind = real_kind_solve), dimension(m,m) :: jacobian
	real(kind = real_kind_solve), dimension(m,1) :: x, x_plus_h, x_minus_h

	real(kind = real_kind_solve) :: h = 10E-8
	integer :: i

	real(kind = real_kind_solve), dimension(m,1) :: f_plus_h, f_minus_h
	real(kind = real_kind_solve), dimension(m,1) :: temp

	do i = 1, m		! This is counting the dimension of x

		!print*, 'Jacobian' , i

		x_plus_h = x
		x_minus_h = x

		x_plus_h(i,1) = x(i,1) + h
		x_minus_h(i,1) = x(i,1) - h

		f_plus_h = f(x_plus_h , m)
		f_minus_h = f(x_minus_h , m)

		temp = (f_plus_h - f_minus_h) / (2. * h)

		jacobian(1:m,i) = temp(1:m,1)

	end do		

end function jacobian

!****************************************************

function norm(input,n,which)

	implicit none

	integer :: n, i, which
	real(kind = real_kind_solve), dimension(n,1) :: input
	real(kind = real_kind_solve) :: norm

	norm = 0

	if (which == 1) then  ! Euclidian Norm

		do i = 1,n
		
			norm = norm + (input(i,1) ** 2)

		end do

		norm = sqrt(norm)

	else if (which == 2) then 
		
		norm = maxval(abs(input))

	else

		print*, 'Error in norm!'

		stop

	end if 


end function norm

!*********************************************************

	SUBROUTINE dludcmp(a,indx,d)
	USE nrtype; USE nrutil, ONLY : assert_eq,imaxloc_dp,nrerror,outerprod,swap_rv_dp
	IMPLICIT NONE
	REAL(8), DIMENSION(:,:), INTENT(INOUT) :: a
	INTEGER(selected_int_kind(9)), DIMENSION(:), INTENT(OUT) :: indx
	REAL(8), INTENT(OUT) :: d
	REAL(8), DIMENSION(size(a,1)) :: vv
	REAL(8), PARAMETER :: TINY=1.0e-20
	INTEGER(selected_int_kind(4)) :: j,n,imax	
	n=assert_eq(size(a,1),size(a,2),size(indx),'dludcmp')
	d=1.0d0
	vv=maxval(abs(a),dim=2)
	if (any(vv == 0.0d0)) call nrerror('singular matrix in dludcmp')
	vv=1.0d0/vv
	do j=1,n
		imax=(j-1)+imaxloc_dp(vv(j:n)*abs(a(j:n,j)))
		if (j /= imax) then
			call swap_rv_dp(a(imax,:),a(j,:))
			d=-d
			vv(imax)=vv(j)
		end if
		indx(j)=imax
		if (a(j,j) == 0.0d0) a(j,j)=TINY
		a(j+1:n,j)=a(j+1:n,j)/a(j,j)
		a(j+1:n,j+1:n)=a(j+1:n,j+1:n)-outerprod(a(j+1:n,j),a(j,j+1:n))
	end do
	END SUBROUTINE dludcmp

!**********************************************************

	SUBROUTINE dlubksb(a,indx,b)
	USE nrtype; USE nrutil, ONLY : assert_eq
	IMPLICIT NONE
	REAL(8), DIMENSION(:,:), INTENT(IN) :: a
	INTEGER(I4B), DIMENSION(:), INTENT(IN) :: indx
	REAL(8), DIMENSION(:), INTENT(INOUT) :: b
	INTEGER(I4B) :: i,n,ii,ll
	REAL(8) :: summ
	n=assert_eq(size(a,1),size(a,2),size(indx),'lubksb')
	ii=0
	do i=1,n
		ll=indx(i)
		summ=b(ll)
		b(ll)=b(i)
		if (ii /= 0) then
			summ=summ-dot_product(a(i,ii:i-1),b(ii:i-1))
		else if (summ /= 0.0d0) then
			ii=i
		end if
		b(i)=summ
	end do
	do i=n,1,-1
		b(i) = (b(i)-dot_product(a(i,i+1:n),b(i+1:n)))/a(i,i)
	end do
	END SUBROUTINE dlubksb



end module